/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: gunzip.cc,v 1.3 2007/03/17 11:50:22 nuno-lopes Exp $
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "gunzip.h"

int gunzip_module(const char *name, const char *tmp)
{
	gzFile file = gzopen(name, "rb");
	if (!file) {
		if (errno == 0) {
			return Z_MEM_ERROR;
		}
		return Z_ERRNO;
	}

	int dest = open(tmp, O_WRONLY | O_CREAT, 0666);
	if (dest == -1) {
		gzclose(file);
		return Z_ERRNO;
	}

	// files opened. now uncompress the data and store it
	// TODO: use mmap() where possible
	do {
		char buf[8192];
		int read = gzread(file, buf, sizeof(buf));

		if (read == 0) { //EOF
			gzclose(file);
			close(dest);
			return Z_OK;

		} else if (read == -1) {
			int err;
			gzerror(file, &err);
			return err;

		} else { //OK, write the buffer to the file
			write(dest, buf, read);
		}

	} while (1);
}
