/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: helper.cc,v 1.5 2007/07/16 13:09:00 nuno-lopes Exp $
 */

#include <iostream>
#include <dlfcn.h>
#include <OPENR/OSyslog.h>
#include "helper.h"

using namespace std;

void* module_executor(void *arg)
{
	BLOCK_SIGNALS()
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	executorArg* args = (executorArg*)arg;

	if (pthread_setspecific(perThreadKey, args->threadData)) {
		OSYSLOG1((osyslogERROR, "pthread_setspecific() failed"));
		return (void*)2;
	}

	void* (*function)(void);

	// load the module and call the entry-point function
	{
	const int dlopen_flags = RTLD_LAZY
#ifdef RTLD_LOCAL
	| RTLD_LOCAL
#endif
#ifdef RTLD_DEEPBIND
	| RTLD_DEEPBIND
#endif
	; // from dlopen_flags

	void *handle = dlopen(args->mod_name.c_str(), dlopen_flags);
	// compatibility with GDB: don't delete the library file because GDB reads the file multiple times
	// unlink(args->mod_name.c_str());

	if (!handle) {
		cerr << "error loading " << dlerror() << endl;
		exit(0x50);
	}

	function = (void* (*)(void))dlsym(handle, "__start_module");
	if (!function) {
		cerr << "Critical error: Couldn't locate the entry point of " << args->mod_name << endl;
		dlclose(handle);
		exit(0x51);
	}
	}

	delete args;

	return function();
}
