/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: images.cc,v 1.8 2007/04/22 09:26:13 nuno-lopes Exp $
 */

#include <cstdlib>
#include "images.h"


/** convert the given image from RGB to YCbCr (each component on its own array) */
void RGB_to_YCbCr(const byte *img, size_t width, size_t height, byte *Y, byte *Cb, byte *Cr)
{
	for (int i=height-1; i >= 0; --i) {
		for (size_t j=0; j < width; ++j) {

		        float B = float(*img++);
		        float G = float(*img++);
		        float R = float(*img++);

			size_t idx = i*width + j;

			Y[idx]  = 16  + char(( 65.738*R + 129.057*G +  25.064*B) * 0.00390625);
			Cb[idx] = 128 + char((-37.945*R -  74.494*G + 112.439*B) * 0.00390625);
			Cr[idx] = 128 + char((112.439*R -  94.154*G -  18.285*B) * 0.00390625);
		}
	}
}


/** shrink an image frame by the given factor */
void shrink_frame(size_t width,
		  size_t height,
		  unsigned int factor,
		  const byte *inY,
		  const byte *inCb,
		  const byte *inCr,
		  byte *outY,
		  byte *outCb,
		  byte *outCr )
{
	size_t i, j, x;

	for (i = x = 0; i < height; i+=factor) {
		for (j = 0; j < width; j+=factor, x++) {
			outY[x]  = inY[i*width+j];
			outCb[x] = inCb[i*width+j];
			outCr[x] = inCr[i*width+j];
		}
	}
}
