/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: MCOOP.cc,v 1.8 2007/07/16 15:18:14 nuno-lopes Exp $
 */

#define _POSIX_C_SOURCE 199309
#include <cstdlib>
#include <ctime>
#include <sys/time.h>
#include <unistd.h>
#include <MCOOP.h>
#include <ObjectEntryTable.h>
#include <MemoryMapAgent.h>
#include <ModuleData.h>

extern "C" {

// this function is called in stub macros and should just return
void Exit(void)
{
	// do nothing
}

void Prologue(InitInfo*)
{
	// do nothing
}

// ---------------------------

sError GetSystemTime(SystemTime* sysTime)
{
	struct timeval tv;

	// TODO: check the original function semantics. is this the time past the boot, the Unix or Aibo epoch time?
	gettimeofday(&tv, NULL);
	sysTime->seconds = tv.tv_sec;
	sysTime->useconds = tv.tv_usec;

	return sSUCCESS;
}


sError SetTimeEvent(TimeEventInfo* info, OID dest, Selector meth, void* msg, size_t sizeOfMsg, EventID* eventID)
{
	// TODO
	return sNOTIMERAVAILABLE;
}

sError CancelEvent(EventID eventID)
{
	// TODO
	return sNOTIMERAVAILABLE;
}

sError NewRegion(size_t size, void** ptr)
{
	*ptr = malloc(size);
	return sSUCCESS;
}

sError DeleteRegion(void* ptr)
{
	free(ptr);
	return sSUCCESS;
}

sError WhoAmI(OID* oid)
{
	oid->SetAddress((void*)MOD_DATA(OID));
	return sSUCCESS;
}

sError NewSharedMemoryRegion(size_t size, MemoryProtectionInfo info, MemoryRegionID* memRegionID, void** baseAddr)
{
	if (info != MEMPROTINFO_R && info != MEMPROTINFO_RW) {
		return sINVALIDSHAREDMEMPROTECT;
	}

	*baseAddr = malloc(size);
	*memRegionID = RegisterNewMemID(*baseAddr);

	// TODO: mark the memory segment as Read-only if asked to do so

	return sSUCCESS;
}

sError DeleteSharedMemoryRegion(MemoryRegionID memRegionID)
{
	if (FreeMemId(memRegionID) == oSUCCESS) {
		return sSUCCESS;
	} else {
		return sINVALIDSHAREDMEMID;
	}
}

sError ChangeProtectionMemoryRegion(MemoryRegionID memRegionID, MemoryProtectionInfo info)
{
	if (info != MEMPROTINFO_R && info != MEMPROTINFO_RW) {
		return sINVALIDSHAREDMEMPROTECT;
	}

	// TODO

	return sSUCCESS;
}

sError GrowMemoryRegion(MemoryRegionID memRegionID, size_t size)
{
	void *addr;

	if (GetMemID(memRegionID, &addr) != oSUCCESS) {
		return sINVALIDSHAREDMEMID;
	}

	// realloc() may change the memory segment and in that case we must return an error, because
	// the original pointer becomes invalid
	if (realloc(addr, size) != addr) {
		return sINVALIDSHAREDMEMID;
	}

	return sSUCCESS;
}

sError ShrinkMemoryRegion(MemoryRegionID memRegionID, size_t size)
{
	return GrowMemoryRegion(memRegionID, size); // use the same implementation
}

sError GetPageSize(size_t *size)
{
	*size = sysconf(_SC_PAGESIZE);
	return sSUCCESS;
}

sError Wait(longword nanosec)
{
	struct timespec t = {0, nanosec};
	nanosleep(&t, NULL);
	return sSUCCESS;
}

}// end of extern "C"
