/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: MemoryMapAgent.cc,v 1.6 2007/01/26 23:12:05 nuno-lopes Exp $
 */

#include <MemoryMapAgent.h>
#include <pthread.h>

/*
	This file implements the database that maps the Memory Region IDs into real addresses
	It shouldn't be used for non-internals stuff
*/

#define LOCK() pthread_mutex_lock(&mutex);
#define UNLOCK() pthread_mutex_unlock(&mutex);

static MemoryRegionID nextMemID = 0;
static map<MemoryRegionID, void*> memIDsDB;
static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;


// temporary implement these functions, as they are automatically called form OSubject and OObject
MemoryMapAgent::MemoryMapAgent() {}
MemoryMapAgent::~MemoryMapAgent() {}


/** get the memory address associated with the specified MemoryRegionID
  returns oFAIL if the memory ID doens't exist, oSUCCESS otherwise */
OStatus GetMemID(MemoryRegionID memID, void** address)
{
	map<MemoryRegionID, void*>::iterator it;

	LOCK();
	it = memIDsDB.find(memID);

	// memID not found
	if (it == memIDsDB.end()) {
		UNLOCK();
		return oFAIL;
	}

	*address = it->second;
	UNLOCK();

	return oSUCCESS;
}


/** Frees the memory associated with the specified memID and remove also
    the associated table entry */
OStatus FreeMemId(MemoryRegionID memID)
{
	map<MemoryRegionID, void*>::iterator it;

	LOCK();
	it = memIDsDB.find(memID);

	// memID not found
	if (it == memIDsDB.end()) {
		UNLOCK();
		return oFAIL;
	}

	void *ptr = it->second;
	memIDsDB.erase(it);
	UNLOCK();

	free(ptr);

	return oSUCCESS;
}


/** Register a new MemoryRegionID. returns the new ID */
MemoryRegionID RegisterNewMemID(void* addr)
{
	LOCK();
	MemoryRegionID id = nextMemID++;
	memIDsDB[id] = addr;
	UNLOCK();

	return id;
}
