/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: ModuleData.h,v 1.10 2007/03/23 20:23:30 nuno-lopes Exp $
 */

#ifndef __ModuleData_h__
#define __ModuleData_h__

#include <OPENR/OPENR.h>
#include <opensdkAPI.h>
#include <pthread.h>
#include <semaphore.h>
#include <map>
#include <queue>
#include <utility>
#include <list>

struct DesignData {
	void *base;
	size_t size;
};

struct message {
	OServiceEntry entry;
	void *msg;
	message(OServiceEntry entry_, void *msg_) : entry(entry_), msg(msg_) {}
};

struct perThreadStruct {
	//for (Find|Delete)DesignData
	std::map<ODesignDataID, DesignData> designDataIDs;
	ODesignDataID lastDesignDataID;

	// semaphore to signalize when there are messages to the object
	sem_t sem;

	// variables that control the shutdown proccess
	pthread_cond_t cond;
	pthread_mutex_t cond_mutex;
	bool exited;

	bool DoInitCompleted;
	bool DoStopCompleted;

	size_t OID;
	pthread_t threadId;

	std::queue<message> queue;
	pthread_mutex_t queue_mutex;

	std::list< std::pair<int,int> > ConnectHandlers;

	char *strtok_ptr; // for our implementation of strtok

	// the constructor
	perThreadStruct(size_t OID_)
	{
		lastDesignDataID = 0;
		OID = OID_;
		exited = false;
		DoInitCompleted = false;
		DoStopCompleted = false;

		sem_init(&sem, 0, 0);
		pthread_cond_init(&cond, NULL);
		pthread_mutex_init(&cond_mutex, NULL);
		pthread_mutex_init(&queue_mutex, NULL);
	}

	// destructor
	~perThreadStruct()
	{
		sem_destroy(&sem);
		pthread_cond_destroy(&cond);
		pthread_mutex_destroy(&cond_mutex);
		pthread_mutex_destroy(&queue_mutex);
	}
};

#define MOD_DATA(x) (((perThreadStruct*)pthread_getspecific(perThreadKey))->x)

#endif
