/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: OFbkImage.cc,v 1.4 2007/04/02 17:45:57 nuno-lopes Exp $
 */

#include <OPENR/OFbkImage.h>
#include <OPENR/OSyslog.h>

OFbkImage::OFbkImage(OFbkImageInfo *info, byte *data, OFbkImageBand band)
{
	width_  = info->width;
	height_ = info->height;
	skip_   = 0;

	switch (band) {
		case ofbkimageBAND_Y:
			image_  = data;
			break;
		case ofbkimageBAND_Cr:
			image_  = data + width_ * height_;
			break;
		case ofbkimageBAND_Cb:
			image_  = data + width_ * height_ * 2;
			break;


		// TODO: implement these
		case ofbkimageBAND_CDT:
		case ofbkimageBAND_Y_LH:
		case ofbkimageBAND_Y_HL:
		case ofbkimageBAND_Y_HH:
			image_ = data;
			break;

		default:
			OSYSLOG1((osyslogERROR, "OFbkImage::OFbkImage: band not understood: %d", band));
	}
}

OFbkImage::~OFbkImage()
{
	// nothing to do
}

OFbkImage& OFbkImage::operator=(const OFbkImage& fbk_image)
{
	image_  = fbk_image.Pointer();
	width_  = fbk_image.Width();
	height_ = fbk_image.Height();
	skip_   = fbk_image.Skip();
	return *this;
}
