/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: OObserver.cc,v 1.18 2007/03/01 22:16:40 nuno-lopes Exp $
 */

#include <iostream>
#include <OPENR/OObserver.h>
#include <opensdkAPI.h>
#include <ModuleData.h>

using namespace std;

#define FETCH_ITERATOR(ret) \
	SubjectConstIterator it = findSubject(id); \
	if (it == end()) { return ret; }


OObserver::OObserver(void) {
	//TODO
	d_skip_ = 0;
	d_min_  = 1;
	d_max_  = 1;
}

OObserver::~OObserver() {
	//TODO
}

OStatus OObserver::SetNotifyEntry(const OServiceEntry& entry)
{
	myID_.Set(entry.GetOID(), entry.GetSelector());

	for (std::list< std::pair<int,int> >::iterator it = MOD_DATA(ConnectHandlers).begin(); it != MOD_DATA(ConnectHandlers).end(); ++it) {
		if (it->second == 0) {
			it->second = entry.GetSelector();
			return oSUCCESS;
		}
	}

	return oFAIL;
}

OStatus OObserver::ConnectHandler(const OConnectMessage& msg, OStatus status)
{
	if (status != oSUCCESS) {
		return status;
	}

	if (msg.msgType != omsgCONNECT) {
		OSYSLOG1((osyslogERROR, "Unknown message type received in ConnectHandler: %d (Entry: %d/%d)", msg.msgType, (size_t)myID_.GetOID().GetAddress(), (int)myID_.GetSelector()));
		return oFAIL;
	}

	switch (msg.command) {
		case oconnectcmdCONNECT:
			{
			OSubjectInfo info(msg.entry, objcommVERSION22);
			subjects_.push_back(info);
			}
			break;

		case oconnectcmdDISCONNECT:
			for (SubjectIterator it = subjects_.begin(); it != subjects_.end(); ++it) {
				if (it->GetSubjectID().GetOID() == msg.entry.GetOID() &&
				it->GetSubjectID().GetSelector() == msg.entry.GetSelector()) {
					subjects_.erase(it);
					break;
				}
			}
			return oFAIL; // subject not found

		case oconnectcmdDISCONNECTALL:
			subjects_.clear();
			break;

		default:
			OSYSLOG1((osyslogERROR, "Unknown message command received in ConnectHandler: %d", msg.command));
			return oFAIL;
	}

	return oSUCCESS;
}

OStatus OObserver::NotifyHandler(const ONotifyMessage& msg, ONotifyEvent* pEvent)
{
	if (msg.msgType != omsgNOTIFY && msg.msgType != omsgNOTIFY_V21 && msg.msgType != omsgNOTIFY_V22) {
		OSYSLOG1((osyslogERROR, "Unknown message type received in NotifyHandler: %d (Entry: %d/%d)", msg.msgType, (size_t)myID_.GetOID().GetAddress(), (int)myID_.GetSelector()));
		return oFAIL;
	}

	pEvent->SetNumOfData(msg.NumberOfData());
	pEvent->SetNumOfNotify(msg.NumberOfNotify());
	pEvent->SetSubjectInfo(OSubjectInfo(msg.subjectID, objcommVERSION22));

	for (int i = 0; i < msg.NumberOfData(); ++i) {
		pEvent->SetData(i, msg.data[i].srcRCR);
	}

	return oSUCCESS;
}

int OObserver::NumberOfSubjects(void) const
{
	return subjects_.size();
}

OStatus OObserver::HandleAddObserver(const OConnectMessage& msg)
{
	//TODO
	return oFAIL;
}

OStatus OObserver::HandleRemoveObserver(const OConnectMessage& msg)
{
	//TODO
	return oFAIL;
}

OStatus OObserver::SendControlMessage(const OConnectMessage& msg, ObjcommCommand command, OControlReplyMessage* replyMsg)
{
	//TODO
	return oFAIL;
}

OStatus OObserver::SendSelfControlMessage(const OConnectMessage& msg, ObjcommCommand command, OControlReplyMessage* replyMsg)
{
	return SendControlMessage(msg, command, replyMsg);
}

OStatus OObserver::SendReadyAll(int cmd)
{
	for (SubjectConstIterator it = begin(); it != end(); ++it){
		if (SendReady(cmd, *it) != oSUCCESS) {
			return oFAIL;
		}
	}

	return oSUCCESS;
}

OStatus OObserver::SendReady(int cmd, const SubjectID& id)
{
	FETCH_ITERATOR(oFAIL)
	return SendReady(cmd, *it);
}

OStatus OObserver::SendReady(int cmd, const OSubjectInfo& info)
{
	OReadyMessage *msg = new OReadyMessage;
	msg->command       = cmd;
	msg->observerID    = myID_;

	return _sendMessage(info.GetSubjectID(), msg);
}

OStatus OObserver::SendReadyV22(int cmd, const SubjectID& id)
{
	return SendReady(cmd, id);
}

OStatus OObserver::SendReadyV10(int cmd, const SubjectID& id)
{
	return SendReady(cmd, id);
}

OStatus OObserver::NotifyHandlerV22(const ONotifyMessage& msg, ONotifyEvent* pEvent)
{
	return NotifyHandler(msg, pEvent);
}

SubjectConstIterator OObserver::findSubject(const ObserverID& id) const
{
	for (SubjectConstIterator it = begin(); it != end(); ++it){
		if (it->GetSubjectID() == id) {
			return it;
		}
	}

	return end(); // invalid iterator
}

SubjectIterator OObserver::findSubject(const SubjectID& id)
{
	for (SubjectIterator it = subjects_.begin(); it != subjects_.end(); ++it){
		if (it->GetSubjectID() == id) {
			return it;
		}
	}

	return subjects_.end(); // invalid iterator
}
