/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: OShmPtrBase.cc,v 1.4 2007/01/26 23:12:05 nuno-lopes Exp $
 */

#include <OPENR/OShmPtrBase.h>
#include <cstring>

OShmPtrBase::OShmPtrBase()
{
	init();
}

OShmPtrBase::OShmPtrBase(const OShmPtrBase& rhs)
{
	pointee_ = rhs.RCRPtr();
	pointee_->AddReference();
}

OShmPtrBase::OShmPtrBase(RCRegion *p)
{
	pointee_ = p;
	pointee_->AddReference();
}

OShmPtrBase::~OShmPtrBase()
{
	Deallocate();
}

void OShmPtrBase::Deallocate()
{
	if (pointee_) {
		pointee_->RemoveReference();
		init();
	}
}

const OShmPtrBase& OShmPtrBase::operator=(const OShmPtrBase& rhs)
{
	Deallocate();
	pointee_ = rhs.RCRPtr();
	pointee_->AddReference();
	return *this;
}

void OShmPtrBase::init()
{
	pointee_ = NULL;
}


// non member function, but also defined in the header
void memcpy(void* s1, const OShmPtrBase& s2, size_t size)
{
	memcpy(s1, s2.Base(), size);
}
