/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: OTime.cc,v 1.5 2007/01/26 23:12:05 nuno-lopes Exp $
 */

#include <OPENR/OTime.h>

OTime::OTime()
{
	clock_ = 0;
	timeDif_ = 0;
}

OTime::OTime(time_t clock, sbyte timeDif)
{
	clock_ = clock;
	timeDif_ = timeDif;
}

OTime::OTime(const OTime& time)
{
	*this = time;
}

OTime& OTime::operator=(const OTime& time)
{
	clock_ = time.clock_;
	timeDif_ = time.timeDif_;
	return *this;
}

bool OTime::operator<(const OTime& time) const
{
	return (clock_ < time.clock_);
}

bool OTime::operator<=(const OTime& time) const
{
	return (clock_ <= time.clock_);
}

bool OTime::operator>(const OTime& time) const
{
	return (clock_ > time.clock_);
}

bool OTime::operator>=(const OTime& time) const
{
	return (clock_ >= time.clock_);
}

bool OTime::operator==(const OTime& time) const
{
	return (clock_ == time.clock_);
}

OStatus OTime::Set(time_t clock, sbyte timeDif)
{
	clock_ = clock;
	timeDif_ = timeDif;
	return oSUCCESS;
}

OStatus OTime::SetClock(time_t clock)
{
	clock_ = clock;
	return oSUCCESS;
}

OStatus OTime::SetLocalTime(time_t localTime)
{
	clock_ = localTime - (timeDif_ * 3600);
	return oSUCCESS;
}

OStatus OTime::SetTimeDif(sbyte timeDif)
{
	timeDif_ = timeDif;
	return oSUCCESS;
}

time_t OTime::GetClock() const
{
	return clock_;
}

time_t OTime::GetLocalTime() const
{
	// time diff represents the time offset (-12 to 12)
	return clock_ + (timeDif_ * 3600);
}

sbyte OTime::GetTimeDif() const
{
	return timeDif_;
}
