/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: ObjcommEvent.cc,v 1.5 2007/01/26 23:12:05 nuno-lopes Exp $
 */

#include <OPENR/ObjcommEvent.h>

OConnectEvent::OConnectEvent(int index, char* name, OConnectCmd command)
{
	obsIndex_ = index;
	subName_  = name;
	command_  = command;
}

OConnectEvent::~OConnectEvent()
{
	// do nothing
}

int OConnectEvent::ObsIndex() const
{
	return obsIndex_;
}

const char* OConnectEvent::SubName() const
{
	return subName_;
}

bool OConnectEvent::IsConnect() const
{
	return (command_ == oconnectcmdCONNECT);
}

bool OConnectEvent::IsDisconnect() const
{
	return (command_ != oconnectcmdCONNECT);
}


OControlEvent::OControlEvent(int index, char* name, ObjcommCommand command)
{
	sbjIndex_ = index;
	subName_  = name;
	command_  = command;
}

OControlEvent::OControlEvent(int index, ObserverID id, char* name, ObjcommCommand command)
{
	sbjIndex_   = index;
	observerID_ = id;
	subName_    = name;
	command_    = command;
}

OControlEvent::~OControlEvent()
{
	// do nothing
}

int OControlEvent::SbjIndex() const
{
	return sbjIndex_;
}

const char* OControlEvent::SubName() const
{
	return subName_;
}

bool OControlEvent::IsAddObserver() const
{
	return (command_ == objcommCMD_ADDOBSERVER || command_ == objcommCMD_ADDOBSERVER_SELF || command_ == objcommCMD_ADDOBSERVER_V22);
}

bool OControlEvent::IsRemoveObserver() const
{
	return (command_ == objcommCMD_REMOVEOBSERVER || command_ == objcommCMD_REMOVEOBSERVER_SELF || command_ == objcommCMD_REMOVEOBSERVER_V22);
}

const ObserverID& OControlEvent::SenderID() const
{
	return observerID_;
}


OReadyEvent::OReadyEvent(int index, ObserverID id, ObjcommCommand command)
{
	sbjIndex_   = index;
	observerID_ = id;
	command_    = command;
}

OReadyEvent::~OReadyEvent()
{
	// nothing to do
}

int OReadyEvent::SbjIndex() const
{
	return sbjIndex_;
}

const ObserverID& OReadyEvent::SenderID() const
{
	return observerID_;
}

bool OReadyEvent::IsAssert() const
{
	return (command_ == objcommCMD_ASSERT_READY);
}

bool OReadyEvent::IsDeassert() const
{
	return (command_ == objcommCMD_DEASSERT_READY);
}


const void* ONotifyEvent::Data(int i) const
{
	return data_[i]->Base();
}

const void** ONotifyEvent::Data() const
{
	return (const void**)data_;
}

RCRegion* ONotifyEvent::RCData(int i) const
{
	return data_[i];
}
