/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: Platform.h,v 1.13 2007/04/25 10:33:21 nuno-lopes Exp $
 */

#ifndef __Platform_h__
#define __Platform_h__

/** the time returned by the time() system call is the number of seconds since 1 Jan 1970
    while the AIBO counts the time since 1 Jan 2000. This is the diffence in seconds between them */
const int TIME_DIFF_SYSTEM = 946684800;

/** time to wait for shutdown (in seconds) */
const int MAX_SHUTDOWN_TIME = 10;

/** time (in microseconds) between each tick (used to update of OVirtualRobotComm data */
const unsigned long USECONDS_TICK = 8000; // 8ms

/** number of ticks between messages to sensor data observers */
const unsigned int NUM_TICKS_TO_SEND_DATA = 4; // 32 ms


#ifdef __USE_GNU
# define opensdk_yield() pthread_yield()
#else
# include <unistd.h>
# if defined(_POSIX_PRIORITY_SCHEDULING) || defined(__CYGWIN__)
#  include <sched.h>
#  define opensdk_yield() sched_yield()
# else
#  warning "defining opensdk_yield() as NOP. this can create some instability. please contact the dev team"
#  define opensdk_yield() /* nothing? */
# endif
#endif


// the robot platform name
#ifdef ERS210
# define ROBOTDESIGN "ERS-210"
#elseif defined(ERS220)
# define ROBOTDESIGN "ERS-220"
#else
# define ROBOTDESIGN "ERS-7"
#endif

// camera image definitions
#ifdef ERS220
const unsigned int IMG_WIDTH  = 176;
const unsigned int IMG_HEIGHT = 144;
#else
const unsigned int IMG_WIDTH  = 208;
const unsigned int IMG_HEIGHT = 160;
#endif

const unsigned int IMG_SIZE   = (IMG_WIDTH * IMG_HEIGHT * 3);
const unsigned int IMG_LAYERS = 4;


// servers socket names
#define AUDIO_SERVER_SOCKET "audioSocket.tmp"
#define ACTUATORS_SERVER_SOCKET "actuatorsSocket.tmp"
#define SENSORS_SERVER_SOCKET "sensorsSocket.tmp"
#define IMAGE_SERVER_SOCKET "imagesSocket.tmp"

#endif
