/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: Primitives.h,v 1.8 2007/02/12 18:57:28 nuno-lopes Exp $
 */

#ifndef __Primitives_h__
#define __Primitives_h__

#include <Types.h>

#define CHECK_PRIMITIVEID() if (primitiveID >= num_of_primitives || primitiveID < 0) { return oINVALID_PRIMITIVE_ID; }
#define CHECK_PRIMITIVEID_ALL() if ((primitiveID >= num_of_primitives || primitiveID < 0) && primitiveID != oprimitiveID_UNDEF) { return oINVALID_PRIMITIVE_ID; }

struct primitiveTable {
	const char *locator;
	word pg, ig, dg, ps, is, ds;
};

// ERS-7 table
// keep'em sorted (because of binary search)
static primitiveTable primitives[] = {
	// CPC Primitive Locator                 PGAIN IGAIN DGAIN PSHIFT ISHOFT SHIFT
	{"PRM:/a1-Sensor:a1",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // accelerometer Y
	{"PRM:/a2-Sensor:a2",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // accelerometer X
	{"PRM:/a3-Sensor:a3",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // accelerometer Z
	{"PRM:/b1-Sensor:b1",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // WLAN switch
	{"PRM:/lu-LED3:lu",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (front, color)
	{"PRM:/lv-LED3:lv",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (front, white)
	{"PRM:/lw-LED3:lw",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (middle, color)
	{"PRM:/lx-LED3:lx",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (middle, white)
	{"PRM:/ly-LED3:ly",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (rear, color)
	{"PRM:/lz-LED3:lz",                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // back light (rear, white)
	{"PRM:/p1-Sensor:p1",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // body PSD
	{"PRM:/r1/c1-Joint2:11",                  0x0A, 0x04, 0x02, 0x0E, 0x02, 0x0F}, // head tilt 1
	{"PRM:/r1/c1/c2-Joint2:12",               0x08, 0x02, 0x04, 0x0E, 0x02, 0x0F}, // head pan
	{"PRM:/r1/c1/c2/c3-Joint2:13",            0x08, 0x04, 0x02, 0x0E, 0x02, 0x0F}, // head tilt 2
	{"PRM:/r1/c1/c2/c3/c4-Joint2:14",         0x08, 0x00, 0x04, 0x0E, 0x02, 0x0F}, // mouth
	{"PRM:/r1/c1/c2/c3/c4/s5-Sensor:s5",      0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // chin switch
	{"PRM:/r1/c1/c2/c3/e5-Joint4:15",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // left ear
	{"PRM:/r1/c1/c2/c3/e6-Joint4:16",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // right ear
	{"PRM:/r1/c1/c2/c3/i1-FbkImageSensor:F1", 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // right ear
	{"PRM:/r1/c1/c2/c3/l1-LED2:l1",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // head light (color)
	{"PRM:/r1/c1/c2/c3/l2-LED2:l2",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // head light (white)
	{"PRM:/r1/c1/c2/c3/l3-LED2:l3",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // mode Indicator (red)
	{"PRM:/r1/c1/c2/c3/l4-LED2:l4",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // mode Indicator (green)
	{"PRM:/r1/c1/c2/c3/l5-LED2:l5",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // mode Indicator (blue)
	{"PRM:/r1/c1/c2/c3/l6-LED2:l6",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // wireless light
	{"PRM:/r1/c1/c2/c3/la-LED3:la",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light1
	{"PRM:/r1/c1/c2/c3/lb-LED3:lb",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light2
	{"PRM:/r1/c1/c2/c3/lc-LED3:lc",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light3
	{"PRM:/r1/c1/c2/c3/ld-LED3:ld",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light4
	{"PRM:/r1/c1/c2/c3/le-LED3:le",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light5
	{"PRM:/r1/c1/c2/c3/lf-LED3:lf",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light6
	{"PRM:/r1/c1/c2/c3/lg-LED3:lg",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light7
	{"PRM:/r1/c1/c2/c3/lh-LED3:lh",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light8
	{"PRM:/r1/c1/c2/c3/li-LED3:li",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light9
	{"PRM:/r1/c1/c2/c3/lj-LED3:lj",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light10
	{"PRM:/r1/c1/c2/c3/lk-LED3:lk",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light11
	{"PRM:/r1/c1/c2/c3/ll-LED3:ll",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light12
	{"PRM:/r1/c1/c2/c3/lm-LED3:lm",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light13
	{"PRM:/r1/c1/c2/c3/ln-LED3:ln",           0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // face light14
	{"PRM:/r1/c1/c2/c3/p1-Sensor:p1",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // head PSD (near)
	{"PRM:/r1/c1/c2/c3/p2-Sensor:p2",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // head PSD (far)
	{"PRM:/r1/c1/c2/c3/t1-Sensor:t1",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // head sensor
	{"PRM:/r2/c1-Joint2:21",                  0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // left front leg J1
	{"PRM:/r2/c1/c2-Joint2:22",               0x14, 0x04, 0x01, 0x0E, 0x02, 0x0F}, // left front leg J2
	{"PRM:/r2/c1/c2/c3-Joint2:23",            0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // left front leg J3
	{"PRM:/r2/c1/c2/c3/c4-Sensor:24",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // left front leg SW
	{"PRM:/r3/c1-Joint2:31",                  0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // left rear leg J1
	{"PRM:/r3/c1/c2-Joint2:32",               0x14, 0x04, 0x01, 0x0E, 0x02, 0x0F}, // left rear leg J2
	{"PRM:/r3/c1/c2/c3-Joint2:33",            0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // left rear leg J3
	{"PRM:/r3/c1/c2/c3/c4-Sensor:34",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // left rear leg SW
	{"PRM:/r4/c1-Joint2:41",                  0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // right front leg J1
	{"PRM:/r4/c1/c2-Joint2:42",               0x14, 0x04, 0x01, 0x0E, 0x02, 0x0F}, // right front leg J2
	{"PRM:/r4/c1/c2/c3-Joint2:43",            0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // right front leg J3
	{"PRM:/r4/c1/c2/c3/c4-Sensor:44",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // right front leg SW
	{"PRM:/r5/c1-Joint2:51",                  0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // right rear leg J1
	{"PRM:/r5/c1/c2-Joint2:52",               0x14, 0x04, 0x01, 0x0E, 0x02, 0x0F}, // right rear leg J2
	{"PRM:/r5/c1/c2/c3-Joint2:53",            0x1C, 0x08, 0x01, 0x0E, 0x02, 0x0F}, // right rear leg J3
	{"PRM:/r5/c1/c2/c3/c4-Sensor:54",         0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // right rear leg SW
	{"PRM:/r6/c1-Joint2:61",                  0x0A, 0x04, 0x04, 0x0E, 0x02, 0x0F}, // tail tilt
	{"PRM:/r6/c2-Joint2:62",                  0x0A, 0x04, 0x04, 0x0E, 0x02, 0x0F}, // tail pan
	{"PRM:/s1-Speaker:S1",                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // speaker
	{"PRM:/t2-Sensor:t2",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // rear back sensor
	{"PRM:/t3-Sensor:t3",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // middle back sensor
	{"PRM:/t4-Sensor:t4",                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // front back sensor
};


#ifdef USARSIM
// USARSim primitives name
static const char* usarsim_primitives[] = {
	NULL, // accelerometer Y
	NULL, // accelerometer X
	NULL, // accelerometer Z
	NULL, // WLAN switch
	NULL, // back light (front, color)
	NULL, // back light (front, white)
	NULL, // back light (middle, color)
	NULL, // back light (middle, white)
	NULL, // back light (rear, color)
	NULL, // back light (rear, white)
	NULL, // body PSD
	"HA", // head tilt 1
	"HB", // head pan
	"HC", // head tilt 2
	NULL, // mouth
	NULL, // chin switch
	NULL, // left ear
	NULL, // right ear
	NULL, // right ear
	NULL, // head light (color)
	NULL, // head light (white)
	NULL, // mode Indicator (red)
	NULL, // mode Indicator (green)
	NULL, // mode Indicator (blue)
	NULL, // wireless light
	NULL, // face light1
	NULL, // face light2
	NULL, // face light3
	NULL, // face light4
	NULL, // face light5
	NULL, // face light6
	NULL, // face light7
	NULL, // face light8
	NULL, // face light9
	NULL, // face light10
	NULL, // face light11
	NULL, // face light12
	NULL, // face light13
	NULL, // face light14
	NULL, // head PSD (near)
	NULL, // head PSD (far)
	NULL, // head sensor
	"LFA", // left front leg J1
	"LFB", // left front leg J2
	"LFC", // left front leg J3
	NULL, // left front leg SW
	"LRA", // left rear leg J1
	"LRB", // left rear leg J2
	"LRC", // left rear leg J3
	NULL, // left rear leg SW
	"RFA", // right front leg J1
	"RFB", // right front leg J2
	"RFC", // right front leg J3
	NULL, // right front leg SW
	"RRA", // right rear leg J1
	"RRB", // right rear leg J2
	"RRC", // right rear leg J3
	NULL, // right rear leg SW
	NULL, // tail tilt
	NULL, // tail pan
	NULL, // speaker
	NULL, // rear back sensor
	NULL, // middle back sensor
	NULL, // front back sensor
};
#endif

const unsigned int num_of_primitives = sizeof(primitives)/sizeof(*primitives);

#endif
