/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: RCRegion.cc,v 1.9 2007/04/25 15:43:10 nuno-lopes Exp $
 */

#include <OPENR/RCRegion.h>
#include <stdlib.h>
#include "MemoryMapAgent.h"

RCRegion::RCRegion()
{
	baseAddr_ = NULL;
	size_ = 0;
	refCounter_ = 0;
	memID_ = 0;
	offset_ = 0;
	status_ = UNDEF;
}

RCRegion::RCRegion(size_t size)
{
	baseAddr_ = (char*)malloc(size);
	size_ = size;
	refCounter_ = 1;
	memID_ = RegisterNewMemID(baseAddr_);
	offset_ = 0;
	status_ = SELF_ALLOCATED;
}

RCRegion::RCRegion(MemoryRegionID memID, offset_t offset, void* baseAddr, size_t size)
{
	memID_ = memID;
	offset_ = offset;
	// TODO check basAddr?
	baseAddr_ = (char*) baseAddr;
	size_ = size;
	refCounter_ = 1;
	status_ = REFERENCE_V20;
}

RCRegion::~RCRegion()
{
	if (IsOwner()) {
		FreeMemId(memID_);
	}
}

void RCRegion::AddReference()
{
	++refCounter_;
}

void RCRegion::RemoveReference()
{
	if (--refCounter_ == 0) {
		delete(this);
	}
}

char* RCRegion::Base() const
{
	return baseAddr_;
}

size_t RCRegion::Size() const
{
	return size_;
}

MemoryRegionID RCRegion::MemID() const
{
	return memID_;
}

offset_t RCRegion::Offset() const
{
	return offset_;
}

RCRegion::RegionType RCRegion::GetType() const
{
	return status_;
}

void RCRegion::SetSize(size_t size)
{
	size_ = size;
}

void* RCRegion::operator new(size_t size)
{
	return ::new RCRegion;
}

void RCRegion::operator delete(void* ptr, size_t size)
{
	::delete (RCRegion*)ptr;
}

bool RCRegion::IsOwner() const
{
	return (GetType() == SELF_ALLOCATED);
}
