/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: libc.cc,v 1.3 2007/07/15 15:06:13 nuno-lopes Exp $
 */

#include <cstring>
#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ModuleData.h>

extern "C" {

/** make strtok thread safe */
char *__wrap_strtok(char *str, const char *delim)
{
	return strtok_r(str, delim, &MOD_DATA(strtok_ptr));
}


/** wrapper around fopen() to make paths relative */
FILE *__wrap_fopen(const char *path, const char *mode)
{
	path = (*path == '/' ? (path+1) : path);
	char *newpath = resolve_case_insensitive_path(path);

	FILE *fp = fopen(newpath, mode);

	free(newpath);
	return fp;
}


/** wrapper around open() to make paths relative */
int __wrap_open(const char *path, int flags, mode_t mode)
{
	path = (*path == '/' ? (path+1) : path);
	char *newpath = resolve_case_insensitive_path(path);

	int fd = open(newpath, flags, mode);

	free(newpath);
	return fd;
}

}
