/*
 * This file is part of openSDK.
 *
 * Copyright (C) 2006-2007 openSDK team
 *
 * openSDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * openSDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with openSDK; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *     $Id: utils.cc,v 1.6 2007/01/26 23:12:05 nuno-lopes Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>
#include <utils.h>
#include <Platform.h>

using namespace std;

/** loads a file to memory. size is filled with the file's size or -1 if the return value is NULL */
const void* file_get_contents(const char *filename, size_t* size)
{
	struct stat sbuf;
	int fd;
	const char *str;

	if ((fd = open(filename, O_RDONLY)) == -1 || fstat(fd, &sbuf) == -1) {
		*size = (size_t)-1;
		return NULL;
	}

	if ((*size = sbuf.st_size) == 0) {
		str = NULL;

	} else if ((str = (const char*)mmap(0, *size, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
		str  = NULL;
		*size = (size_t)-1;
	}

	close(fd);
	return str;
}

/** frees the memory allocated with file_get_contents */
bool file_unmap_contents(void* base, size_t size)
{
	return (munmap(base, size) == 0 ? true : false);
}

/** wrapper around STL's getline. this only returns non-empty line plus trimming and optional robot model checking */
bool getNonEmptyLine(fstream& file, string& line, bool onlyMySection, bool* mySection)
{
	while(getline(file, line)) {
		size_t idx = line.find_first_not_of(" \t");
		if (idx != string::npos && idx != 0) {
			line.erase(0, idx);
		}

		// skip comments and blank lines
		if (line[0] == '#' || line[0] == '\r' || line[0] == '\n') {
			continue;
		}

		if (onlyMySection) {
			if (line[0] == '[') { // start of a new section
				*mySection = (line == "[" ROBOTDESIGN "]");
				continue;
			} else if(!*mySection) {
				continue;
			}
		}

		size_t end = line.find_first_of("\r\n");
		if (end != string::npos) {
			line[end] = '\0';
		}

		// skip empty lines
		if (end == 0 || !line[0]) {
			continue;
		}

		return true;
	}

	return false;
}
